var ALL_PROPERTIES_NOT_FOUND = new Error('All Properties Not Found');
var defaultObject = Object.freeze(Object.create(null));
/**
 * Uses ES2015 Proxy to partially apply a function that takes in an options object
 * @param f :: Object a => (a -> b)
 */
export var curryObj = function (f) {
    return _curryObj(f, defaultObject);
};
function _curryObj(f, previousObj) {
    return function (x) {
        var obj = proxyObject(Object.assign({}, previousObj, x));
        try {
            return f(obj);
        }
        catch (error) {
            if (error !== ALL_PROPERTIES_NOT_FOUND) {
                throw error;
            }
            return _curryObj(f, obj);
        }
    };
}
function proxyObject(a) {
    return new Proxy(a, {
        get: function (target, key) {
            if (!target.hasOwnProperty(key)) {
                throw ALL_PROPERTIES_NOT_FOUND;
            }
            return target[key];
        },
    });
}
//# sourceMappingURL=curryObj.js.map